@@ -1,5 +1,8 @@
 """Defines class Project."""
 
+import pickle
+import codecs
+
 import reapy
 from reapy import reascript_api as RPR
 from reapy.core import ReapyObject
@@ -16,14 +19,24 @@ class Project(ReapyObject):
 
         Parameters
         ----------
-        id : str, optional
-            Project ID. If None, `index` must be specified.
+        id : None, str or int, optional
+            Project identifier.
+            When None (default), `index is used instead.
+            An integer is interpreted as the project index in GUI.
+            A string starting with '(ReaProject*)0x' is interpreted
+            as a ReaScript identifier.
+            Otherwise, `id` is the project name. In that case, the .rpp
+            extension is optional.
         index : int, optional
             Project index in GUI (default=-1, corresponds to current
             project).
         """
+        if isinstance(id, int):
+            id, index = None, id
         if id is None:
             id = RPR.EnumProjects(index, None, 0)[0]
+        if not id.startswith('(ReaProject*)0x'):
+            id = Project._from_name(id).id
         self.id = id
 
     def __eq__(self, other):
@@ -35,6 +48,34 @@ class Project(ReapyObject):
     def _args(self):
         return self.id,
 
+    @staticmethod
+    def _from_name(name):
+        """Return project with corresponding name.
+
+        Parameters
+        ----------
+        name : str
+            Project file name. Including the extension ('.rpp')
+            is optional.
+
+        Returns
+        -------
+        Project
+
+        Raises
+        ------
+        NameError
+            If no project with the corresponding name is open.
+        """
+        if not name.lower().endswith('.rpp'):
+            name += '.rpp'
+        with reapy.inside_reaper():
+            for project in reapy.get_projects():
+                project_name = project.name[:-4] + '.rpp'
+                if project_name == name:
+                    return project
+        raise NameError('"{}" is not currently open.'.format(name))
+
     @reapy.inside_reaper()
     def _get_track_by_name(self, name):
         """Return first track with matching name."""
@@ -325,6 +366,68 @@ class Project(ReapyObject):
         take = item.takes[res[3] // 2**16]
         return take.fxs[res[3] % 2**16]
 
+    def get_info_string(self, param_name: str) -> str:
+        """
+        Parameters
+        ----------
+        param_name : str
+            MARKER_GUID:X : get the GUID (unique ID) of the marker or region
+                with index X, where X is the index passed to
+                EnumProjectMarkers, not necessarily the displayed number
+            RECORD_PATH :
+                recording directory -- may be blank or a relative path,
+                to get the effective path see GetProjectPathEx()
+            RENDER_FILE : render directory
+            RENDER_PATTERN : render file name (may contain wildcards)
+            RENDER_FORMAT : base64-encoded sink configuration
+                (see project files, etc). Callers can also pass a simple
+                4-byte string (non-base64-encoded), e.g. "evaw" or "l3pm",
+                to use default settings for that sink type.
+            RENDER_FORMAT2 : base64-encoded secondary sink configuration.
+                Callers can also pass a simple 4-byte string (non-base64-encoded),
+                e.g. "evaw" or "l3pm", to use default settings for
+                that sink type, or "" to disable secondary render.
+                Formats available on this machine:
+                "wave" "aiff" "iso " "ddp " "flac" "mp3l" "oggv" "OggS"
+                "FFMP" "GIF " "LCF " "wvpk"
+        """
+        _, _, _, result, _ = RPR.GetSetProjectInfo_String(
+            self.id, param_name, 'valuestrNeedBig', False
+        )
+        return result
+
+    def get_info_value(self, param_name: str) -> float:
+        """
+        Parameters
+        ----------
+        param_name : str
+            RENDER_SETTINGS : &(1|2)=0:master mix, &1=stems+master mix,
+                &2=stems only, &4=multichannel tracks to multichannel files,
+                &8=use render matrix, &16=tracks with only mono media
+                to mono files, &32=selected media items,
+                &64=selected media items via master
+            RENDER_BOUNDSFLAG : 0=custom time bounds, 1=entire project,
+                2=time selection, 3=all project regions,
+                4=selected media items, 5=selected project regions
+            RENDER_CHANNELS : number of channels in rendered file
+            RENDER_SRATE : sample rate of rendered file
+                (or 0 for project sample rate)
+            RENDER_STARTPOS : render start time when RENDER_BOUNDSFLAG=0
+            RENDER_ENDPOS : render end time when RENDER_BOUNDSFLAG=0
+            RENDER_TAILFLAG : apply render tail setting when rendering:
+                &1=custom time bounds, &2=entire project, &4=time selection,
+                &8=all project regions, &16=selected media items,
+                &32=selected project regions
+            RENDER_TAILMS : tail length in ms to render
+                (only used if RENDER_BOUNDSFLAG and RENDER_TAILFLAG are set)
+            RENDER_ADDTOPROJ : 1=add rendered files to project
+            RENDER_DITHER : &1=dither, &2=noise shaping, &4=dither stems,
+                &8=noise shaping on stems
+            PROJECT_SRATE : samplerate (ignored unless PROJECT_SRATE_USE set)
+            PROJECT_SRATE_USE : set to 1 if project samplerate is used
+        """
+        return RPR.GetSetProjectInfo(self.id, param_name, None, False)
+
     def get_play_rate(self, position):
         """
         Return project play rate at a given position.
@@ -383,6 +486,27 @@ class Project(ReapyObject):
         track = reapy.Track(track_id)
         return track
 
+    def get_ext_state(self, section, key, pickled=False):
+        """
+        Return external state of project.
+
+        Parameters
+        ----------
+        section : str
+        key : str
+        pickled: bool
+            Whether data was pickled or not.
+
+        Returns
+        -------
+        value : str
+            If key or section does not exist an empty string is returned.
+        """
+        value = RPR.GetProjExtState(self.id, section, key, "", 2**31 - 1)[4]
+        if value and pickled:
+            value = pickle.loads(codecs.decode(value.encode(), "base64"))
+        return value
+
     def glue_items(self, within_time_selection=False):
         """
         Glue items (action shortcut).
@@ -804,7 +928,7 @@ class Project(ReapyObject):
         """
         return [
             reapy.Item(RPR.GetSelectedMediaItem(self.id, i))
-            for i in range(self.n_items)
+            for i in range(self.n_selected_items)
         ]
 
     @reapy.inside_reaper()
@@ -826,6 +950,102 @@ class Project(ReapyObject):
         for track in tracks:
             track.select()
 
+    def set_info_string(self, param_name, param_string):
+        """
+        Parameters
+        ----------
+        param_name : str
+            MARKER_GUID:X : get the GUID (unique ID) of the marker or region
+                with index X, where X is the index passed to
+                EnumProjectMarkers, not necessarily the displayed number
+            RECORD_PATH :
+                recording directory -- may be blank or a relative path,
+                to get the effective path see GetProjectPathEx()
+            RENDER_FILE : render directory
+            RENDER_PATTERN : render file name (may contain wildcards)
+            RENDER_FORMAT : base64-encoded sink configuration
+                (see project files, etc). Callers can also pass a simple
+                4-byte string (non-base64-encoded), e.g. "evaw" or "l3pm",
+                to use default settings for that sink type.
+            RENDER_FORMAT2 : base64-encoded secondary sink configuration.
+                Callers can also pass a simple 4-byte string (non-base64-encoded),
+                e.g. "evaw" or "l3pm", to use default settings for
+                that sink type, or "" to disable secondary render.
+                Formats available on this machine:
+                "wave" "aiff" "iso " "ddp " "flac" "mp3l" "oggv" "OggS"
+                "FFMP" "GIF " "LCF " "wvpk"
+        param_string : str
+        """
+        RPR.GetSetProjectInfo_String(
+            self.id, param_name, param_string, True
+        )
+
+    def set_info_value(self, param_name, param_value):
+        """
+        Parameters
+        ----------
+        param_name : str
+            RENDER_SETTINGS : &(1|2)=0:master mix, &1=stems+master mix,
+                &2=stems only, &4=multichannel tracks to multichannel files,
+                &8=use render matrix, &16=tracks with only mono media
+                to mono files, &32=selected media items,
+                &64=selected media items via master
+            RENDER_BOUNDSFLAG : 0=custom time bounds, 1=entire project,
+                2=time selection, 3=all project regions,
+                4=selected media items, 5=selected project regions
+            RENDER_CHANNELS : number of channels in rendered file
+            RENDER_SRATE : sample rate of rendered file
+                (or 0 for project sample rate)
+            RENDER_STARTPOS : render start time when RENDER_BOUNDSFLAG=0
+            RENDER_ENDPOS : render end time when RENDER_BOUNDSFLAG=0
+            RENDER_TAILFLAG : apply render tail setting when rendering:
+                &1=custom time bounds, &2=entire project, &4=time selection,
+                &8=all project regions, &16=selected media items,
+                &32=selected project regions
+            RENDER_TAILMS : tail length in ms to render
+                (only used if RENDER_BOUNDSFLAG and RENDER_TAILFLAG are set)
+            RENDER_ADDTOPROJ : 1=add rendered files to project
+            RENDER_DITHER : &1=dither, &2=noise shaping, &4=dither stems,
+                &8=noise shaping on stems
+            PROJECT_SRATE : samplerate (ignored unless PROJECT_SRATE_USE set)
+            PROJECT_SRATE_USE : set to 1 if project samplerate is used
+        param_value : float
+        """
+        RPR.GetSetProjectInfo(self.id, param_name, param_value, True)
+
+    def set_ext_state(self, section, key, value, pickled=False):
+        """
+        Set external state of project.
+
+        Parameters
+        ----------
+        section : str
+        key : str
+        value : Union[Any, str]
+            State value. Will be dumped to str using either `pickle` if
+            `pickled` is `True` or `json`. Length of the dumped value
+            must not be over 2**31 - 2.
+        pickled : bool, optional
+            Data will be pickled with the last version if True.
+            If you using mypy as type checker, typing_extensions.Literal[True]
+            has to be used for `pickled`.
+
+        Raises
+        ------
+        ValueError
+            If dumped `value` has length over 2**31 - 2.
+        """
+        if pickled:
+            value = pickle.dumps(value)
+            value = codecs.encode(value, "base64").decode()
+        if len(value) > 2**31 - 2:
+            message = (
+                "Dumped value length is {:,d}. It must not be over "
+                "2**31 - 2."
+            )
+            raise ValueError(message.format(len(value)))
+        RPR.SetProjExtState(self.id, section, key, value)
+
     @reapy.inside_reaper()
     def solo_all_tracks(self):
         """
